package Source::Package::Comparators;

use strict;
use warnings;
use Carp;
use RPM::Vercmp;
use Source::Package::Comparators::Raw;

our $VERSION=0.002;
our $verbose=0;

sub null_comparator {
    my ($origin, $dest)=@_;
    return 0;
}

sub null_epoch_comparator {
    my ($origin, $dest)=@_;
    return 0;
}

sub generic_epoch_comparator {
    my ($origin, $dest)=@_;
    return $origin->epoch <=> $dest->epoch;
}

sub rpm40_epoch_comparator {
    my ($origin, $dest)=@_;
    return &__rpm40epoch($origin->epoch) <=> &__rpm40epoch($dest->epoch);
}

sub rpmorg_epoch_comparator {
    my ($origin, $dest)=@_;
    return &__rpmorg_epoch($origin->epoch) <=> &__rpmorg_epoch($dest->epoch);
}

sub __rpm40epoch {
    return $_[0] ? $_[0]: 0;
}

sub __rpmorg_epoch {
    return (defined($_[0]) && $_[0] ne '') ? $_[0]: -1;
}

sub pure_rpm_version_comparator {
    my ($origin, $dest)=@_;
    return RPM::Vercmp::vercmp($origin->version, 0, $dest->version, 0);
}

sub pure_rpm_release_comparator {
    my ($origin, $dest)=@_;
    return RPM::Vercmp::vercmp(0, $origin->release, 0, $dest->release);
}

sub null_release_comparator {
    my ($origin, $dest)=@_;
    return 0;
}

sub rpm_plus_perl_version_comparator {
    my ($origin, $dest)=@_;
    return &Source::Package::Comparators::Raw::_rpm_plus_perl_version_cmp($origin->name,$dest->name,$origin->version, $dest->version);
}

1;
