package Source::Package::Comparators::Raw;

use strict;
use warnings;
use Carp;
use version;
use RPM::Vercmp;

our $VERSION=0.007;
our $verbose=0;

# used in ImportSkipMassRebuild, Mass::ALTLinuxImport
sub import_trim_release {
    my $rel=shift;
    # 1.mga7.nonfree 3.mga7.tainted 1.fc24.R (russianfedora)
    # (?=(?:\.|$)) due to smth like Release: 7%{?dist}.1
    $rel=~s/\.(?:fc|mga)\d+(?:\.(?:R|nonfree|tainted))?(?=(?:\.|$))//;
    return $rel;
}

###### PROTECTED #############################################

# used in in watch-rear
sub _suse_enhanced_rpm_plus_perl_version_cmp {
    my ($originname, $destname, $originversion, $destversion)=@_;
    Carp::confess 'Oops!: undefined versions' unless defined $destversion and defined $originversion;
    &__clean_suse_enhanced1(\$originversion,\$destversion);
    &__clean_suse_enhanced1(\$destversion,\$originversion);
    return &_rpm_plus_perl_version_cmp($originname, $destname,$originversion,$destversion);
}

sub __clean_suse_enhanced1 {
    my ($ver1ref,$ver2ref)=@_;
    # 3.0.7+git20151023.72.a0ff256
    $$ver1ref=~s/\+.*$//;
    # 3.0.7~rc3
    $$ver1ref=~s/~rc.*$// if $$ver2ref !~ /~rc.*$/;
    # 1.4.1.2~git0.9a126614a
    $$ver1ref=~s/~git.*$// if $$ver2ref !~ /~git.*$/;
    # 0.10.0.svn20080724
    $$ver1ref=~s/\.(?:svn|git).*$// if $$ver2ref !~ /\.(?:svn|git).*$/;
    # 1.42_bld115
    $$ver1ref=~s/_bld.*$// if $$ver2ref !~ /_bld.*$/;
    # ver.r1
    $$ver1ref=~s/\.[rR]\d+$// if $$ver2ref !~ /\.[rR]\d+$/;
    # rc3
    $$ver1ref=~s/\.?rc\d+$//i if $$ver2ref !~ /\.?rc\d+$/i;
    # 1.0b3
    $$ver1ref=~s/[abrABR]\d+$// if $$ver2ref !~ /[abrABR]\d+$/;
    # 1.0c
    $$ver1ref=~s/[a-zA-Z]$// if $$ver1ref =~ /\d[a-zA-Z]$/ and $$ver2ref !~ /\d[a-zA-Z]$/;
    # 0.20080710 vs 20020904
    $$ver1ref=~s/^0\.// if $$ver1ref =~ /^0\.(?:19|20)\d{6}$/ and $$ver2ref =~ /^(?:19|20)\d{6}$/;
    # 2018 vs 20180904
    $$ver1ref=~s/^(\d\d\d\d)\d\d\d\d/$1/ if $$ver1ref =~ /^20[0123]\d{5}$/ and $$ver2ref =~ /^20[0123]\d$/;
    # build19 vs 0
    $$ver1ref='0' if $$ver1ref =~ /^build\d+$/ and $$ver2ref eq '0';

}

# used in ALTLinuxImport, in RaiseRelease, in watch-rear
sub _rpm_plus_perl_version_cmp {
    my ($originname, $destname, $originversion, $destversion)=@_;
    Carp::confess 'Oops!: undefined versions' unless defined $destversion and defined $originversion;
    $originname='(undef)' if not defined $originname;
    if ($destname =~/^perl-/) {
	my $compare_result;
	eval {
	    $compare_result = &perl_version_cmp($originversion, $destversion);
	    };
	die "_rpm_plus_perl_version_cmp: $originname, $destname, $originversion, $destversion: $@.\n" if $@;
	return $compare_result;
    }
    return &__zero_normalize($originversion,$destversion,
			     RPM::Vercmp::vercmp($originversion, 0, $destversion, 0));
}

sub perl_version_cmp {
    my ($originversion,$destversion)=@_;
    Carp::confess 'incorrect call: 3-rd,4-th args found!' if $_[2] or $_[3];
    return &__zero_normalize($originversion,$destversion,
			     &__raw_perl_version_cmp($originversion,$destversion));
}

sub __zero_normalize {
    my ($originversion,$destversion,$cmp)=@_;
    $cmp=0 if $cmp>0 and $destversion.'.0' eq $originversion; # like, 1.4 equal 1.4.0
    $cmp=0 if $cmp<0 and $originversion.'.0' eq $destversion; # like, 1.4 equal 1.4.0
    return $cmp;
}

use Source::Package::Tarball::Python::Version;
# who will use it?
sub python_version_cmp {
    my ($originversion,$destversion)=@_;
    Carp::confess 'incorrect call: 3-rd,4-th args found!' if $_[2] or $_[3];
    my $v1=Source::Package::Tarball::Python::Version->parse($originversion);
    my $v2=Source::Package::Tarball::Python::Version->parse($destversion);
    my $cmp=Source::Package::Tarball::Python::Version::cmp($v1, $v2);
    return $cmp;
}

###### PRIVATE #############################################

sub __raw_perl_version_cmp {
    my ($ver1, $ver2)=@_;
    my ($v1s,$tail1)=&__raw_perl_ver_trim($ver1);
    my ($v2s,$tail2)=&__raw_perl_ver_trim($ver2);
    my $v1=version->parse($v1s);
    my $v2=version->parse($v2s);

    if ($v1 == $v2) {
	if (not $tail1 and not $tail2) {
	    return 0;
	} else {
	    #warn "tails found in $v1s $v2s\n";
	    if ($tail2 and not $tail1 and $tail2=~/[-\._]?(alpha|beta)\d$/) {
		# 1.0 > 1.0beta3
		return 1;
	    } else {
		return RPM::Vercmp::evrcmp($tail1, $tail2);
	    }
	}
    }

    #warn "methods differ! $v1s, $v2s\n" if (evrcmp($s1, $s2) < 0) xor ($v1 < $v2);
    return $v1 cmp $v2;
}

sub __raw_perl_ver_trim {
    my $vers=shift;
    $vers=~s/^V/v/;
    # Curses-Simp-1.4.A8UG1gG, etc 
    $vers=~s/^(v?[\d\.]*\d)\.?([^\d\.].*)?$/$1/;
    my $tail=$2;
    $tail||='';
    return ($vers,$tail);
}

1;
