package Source::Package::RPM;

use strict;
use warnings;
use RPM::Header;
use Source::Package;
our @ISA=qw/Source::Package/;
use Source::Shared::Utils::Downloader qw/cached_download/;

our $VERSION=0.002;

use Source::Package::Comparators;
sub get_raw_epoch_comparator {
    return \&Source::Package::Comparators::rpmorg_epoch_comparator;
}

sub is_pkg {
   my (undef,$item)=@_;
   return if not (defined $item and -e $item and not -d $item);
   return $item=~/\.src\.rpm$/;
}

sub file2pkg {
    my ($self,$file)=@_;
    my $rh = new RPM::Header $file;
    die "file2pkg: can't get RPM::Header: strange rpm file: $file" unless $rh;
    return $self->_rpmheader2pkg($rh);
}

sub url2pkg {
    my ($self,$url,$cachedir)=@_;
    $cachedir//=$self->{-cachedir} if $self;
    my $file=cached_download($url, $cachedir);
    my $rh = new RPM::Header $file;
    die "file2pkg: can't get RPM::Header: strange rpm file: $file" unless $rh;
    return $self->_rpmheader2pkg($rh);
}

# protected;
# overridden in subclasses
sub _rpmheader2pkg {
    my ($self,$rh)=@_;
    return $self->new(
	-name => $rh->{NAME},
	-version => $rh->{VERSION},
	-release => $rh->{RELEASE},
	-epoch => $rh->{EPOCH},
	-file => $rh->source_name,
	);
}


### object method
# sub pkg4rpmeditor {
#     my $self=shift;
#     return [ SOURCERPM => $self->file];
# }

1;
