package Source::Package::RPM::ALT;

use strict;
use warnings;
use RPM::Header;
use Source::Package::RPM;
our @ISA=qw/Source::Package::RPM/;
use Source::Shared::Utils::Downloader qw/cached_download/;

our $VERSION=0.001;

use Source::Package::Comparators;
sub get_raw_epoch_comparator {
    return \&Source::Package::Comparators::rpmorg_epoch_comparator;
}

sub is_pkg {
   my (undef,$item)=@_;
   return if not (defined $item and -e $item and not -d $item);
   return $item=~/\.(?:tar|src\.rpm)$/;
}

sub file2pkg {
    my ($self,$file)=@_;
    if ($file=~m!(?:^|/)([^/]+)-([^-/]+)-([^-/]+)\.tar$!) {
	return $self->new(
	    -name => $1,
	    -version => $2,
	    -release => $3,
	    -file => $file,
	    );
    }
    my $rh = new RPM::Header $file;
    die "file2pkg: can't get RPM::Header: strange rpm file: $file" unless $rh;
    return $self->_rpmheader2pkg($rh);
}

sub url2pkg {
    my ($self,$url,$cachedir)=@_;
    $cachedir//=$self->{-cachedir} if $self;
    my $file=cached_download($url, $cachedir);
    if ($url=~m!(?:^|/)([^/]+)-([^-/]+)-([^-/]+)\.tar(?:$|\?)!) {
	return $self->new(
	    -name => $1,
	    -version => $2,
	    -release => $3,
#	    -url => $url,
	    -file => $file,
	    );
    }
    my $rh = new RPM::Header $file;
    die "file2pkg: can't get RPM::Header: strange rpm file: $file" unless $rh;
    return $self->_rpmheader2pkg($rh);
}

1;
