package Source::Package::Tarball::Perl;

use strict;
use warnings;

use File::Basename;
use Source::Package::Tarball;

our @ISA=qw/Source::Package::Tarball/;

our $VERSION=0.002;

our $verbose;

sub trim_package_version {
    my $class=shift;
    return &_cpan_trim_version;
}

sub guess_package_name_version {
    my $class=shift;
    return &_cpan_guess_pkg_name_version;
}

#------------------------------------------------------
# shared static CPAN maintainance code
#------------------------------------------------------

#------------------------------------------------------
my %KNOWN_NUMERICS=map{$_=>1} qw!
ACME-Error-31337
SWISH-3
Acme-123
Acme-1337
Acme-24
Acme-6502
Acme-CPANLists-Import-PerlAdvent-2000
Acme-CPANLists-Import-PerlAdvent-2001
Acme-CPANLists-Import-PerlAdvent-2002
Acme-CPANLists-Import-PerlAdvent-2003
Acme-CPANLists-Import-PerlAdvent-2004
Acme-CPANLists-Import-PerlAdvent-2005
Acme-CPANLists-Import-PerlAdvent-2006
Acme-CPANLists-Import-PerlAdvent-2007
Acme-CPANLists-Import-PerlAdvent-2008
Acme-CPANLists-Import-PerlAdvent-2009
Acme-CPANLists-Import-PerlAdvent-2010
Acme-CPANLists-Import-PerlAdvent-2011
Acme-CPANLists-Import-PerlAdvent-2012
Acme-CPANLists-Import-PerlAdvent-2013
Acme-CPANLists-Import-PerlAdvent-2014
Acme-CPANLists-Import-PerlAdvent-2015
Acme-CPANLists-Import-PerlAdvent-2016
Acme-CPANLists-Import-PerlAdvent-2017
Acme-CPANLists-Import-PerlAdvent-2018
Acme-CPANLists-Import-PerlAdvent-2019
Acme-CPANLists-Import-PERLANCAR-Advent-2014
Acme-CPANLists-Import-PerlDancerAdvent-2010
Acme-CPANLists-Import-PerlDancerAdvent-2011
Acme-CPANLists-Import-PerlDancerAdvent-2012
Acme-CPANLists-Import-PerlDancerAdvent-2014
Acme-CPANLists-Import-PerlDancerAdvent-2015
Acme-CPANLists-Import-PerlDancerAdvent-2016
Acme-CPANLists-Import-PerlDancerAdvent-2017
Acme-CPANLists-Import-PerlDancerAdvent-2018
Acme-CPANLists-Import-PerlDancerAdvent-2019
Acme-CPANModulesBundle-Import-MojoliciousAdvent-2017
Acme-CPANModulesBundle-Import-MojoliciousAdvent-2018
Acme-CPANModulesBundle-Import-MojoliciousAdvent-2019
Acme-CPANModulesBundle-Import-PERLANCAR-Advent-2014
Acme-CPANModulesBundle-Import-PerlAdvent-2000
Acme-CPANModulesBundle-Import-PerlAdvent-2001
Acme-CPANModulesBundle-Import-PerlAdvent-2002
Acme-CPANModulesBundle-Import-PerlAdvent-2003
Acme-CPANModulesBundle-Import-PerlAdvent-2004
Acme-CPANModulesBundle-Import-PerlAdvent-2005
Acme-CPANModulesBundle-Import-PerlAdvent-2006
Acme-CPANModulesBundle-Import-PerlAdvent-2007
Acme-CPANModulesBundle-Import-PerlAdvent-2008
Acme-CPANModulesBundle-Import-PerlAdvent-2009
Acme-CPANModulesBundle-Import-PerlAdvent-2010
Acme-CPANModulesBundle-Import-PerlAdvent-2011
Acme-CPANModulesBundle-Import-PerlAdvent-2012
Acme-CPANModulesBundle-Import-PerlAdvent-2013
Acme-CPANModulesBundle-Import-PerlAdvent-2014
Acme-CPANModulesBundle-Import-PerlAdvent-2015
Acme-CPANModulesBundle-Import-PerlAdvent-2016
Acme-CPANModulesBundle-Import-PerlAdvent-2017
Acme-CPANModulesBundle-Import-PerlAdvent-2018
Acme-CPANModulesBundle-Import-PerlAdvent-2019
Acme-CPANModulesBundle-Import-PerlDancerAdvent-2010
Acme-CPANModulesBundle-Import-PerlDancerAdvent-2011
Acme-CPANModulesBundle-Import-PerlDancerAdvent-2012
Acme-CPANModulesBundle-Import-PerlDancerAdvent-2013
Acme-CPANModulesBundle-Import-PerlDancerAdvent-2014
Acme-CPANModulesBundle-Import-PerlDancerAdvent-2015
Acme-CPANModulesBundle-Import-PerlDancerAdvent-2016
Acme-CPANModulesBundle-Import-PerlDancerAdvent-2017
Acme-CPANModulesBundle-Import-PerlDancerAdvent-2018
Acme-CPANModulesBundle-Import-PerlDancerAdvent-2019
Acme-CPANModules-NewDistributions-202001
Acme-Test-42
App-1234567891
App-12567834
App-Bin4TSV-6
App-Bin4TSV-8
App-Bin4TSV-9
ACME-YAPC-NA-2012
Chess-960
Class-DBI-Frozen-301
Games-2048
ISO-639
ISO-639-3
ISO-639-5
Lingua-31337
Perl-Dist-Strawberry-BuildPerl-5123
Perl-Dist-WiX-BuildPerl-5123
Pod-Simple-31337
POSIX-1003
POSIX-2008
TableData-Perl-CPAN-Release-Static-2021
TableData-Perl-CPAN-Release-Static-2022
TableData-Perl-CPAN-Release-Static-2023
TableData-Perl-CPAN-Release-Static-2024
TableData-Perl-CPAN-Release-Static-2025
Text-Numbers-100
tinyperl-1.0-580
WordList-Test-Dynamic-Number-10000
WordList-Test-Dynamic-RandomWord-1000
!;
my %KNOWN_ALPHABETIC=map{$_=>1} qw!
BGPmon-Analytics-db
BGPmon-Archiver
BGPmon-Client
BGPmon-core
BGPmon-CPM
BGPmon-CPM-Prefix-Finder
BGPmon-Filter
BGPmon-Validate
Math-GoldenBigMath
Math-MatrixReal-Ext1
Model3D-WavefrontObject
PDF-CreateSimple
!;
my %KNOWN_DOT_SEPARATOR=map{$_=>1} qw!
AI-ExpertSystem-Simple
FLAT-Legacy-FA
!;
my %KNOWN_EXCEPTIONS=(
    'Fraction-v.53b' => ['Fraction', '0.53b'],
    'WWW-TMDB-API0.03' => ['WWW-TMDB-API', '0.03'],
    'WWW-TMDB-API0.04' => ['WWW-TMDB-API', '0.04'],
    'XMS-MotifSetv1.0' => ['XMS-MotifSet','v1.0'],
    'Win32-TaskScheduler2.0.3' => ['Win32-TaskScheduler','2.0.3'],
    'Win32-Exchange_v0.046a' => ['Win32-Exchange','v0.046a']
    );

#------------------------------------------------------

sub _cpan_guess_pkg_name_version {
    my ($pkg)=@_;
    unless ($pkg=~s/\.(?:tar.gz|tar.bz2|tgz|zip)$//) {
	warn "WARN: cpan_guess_pkg_name_version: not a package: $pkg\n";
	return;
    }
    my $module=basename($pkg);
    my $ver;
    my $separator;
    if ($module=~s/-([^-]+)$//) {
	$ver=$1;
	$separator='-';
	print STDERR "cpan_guess:1: sep=$separator mod=$module ver=$ver\n" if $verbose;
	#HTML-WikiConverter-DokuWikiFCK-0.32_Feb-27-2013
	if ($ver=~/^20[12]\d$/ and $module=~s/-([\d\.]+_\w+[-\.]\d\d)$//) {
	    $ver=$1.$separator.$ver;
	} elsif (not $KNOWN_NUMERICS{$module} and $module=~s/-([\d\.]+)$//) {
	    my $prefix=$1;
	    if ($prefix=~/\d\.\d/ or $KNOWN_ALPHABETIC{$module}) {
		$ver="$prefix$separator$ver";
	    } else {
		$module.=$separator.$prefix;
		warn "cpan_guess_pkg_name_version: WARN: name_with_digits: $pkg $module $ver\n";
	    }
	}
    } elsif ($module=~s/[a-zA-Z]\.(\d[\d\.]+)$//) {
	$ver=$1;
	$separator='.';
	print STDERR "cpan_guess:2: sep=$separator mod=$module ver=$ver\n" if $verbose;
    } elsif ($module=~s/_([^_]+)(?:_(?:stable|alpha|bis|withoutworldwriteables|source|OpenSource))?$//) {
	$ver=$1;
	$separator='_';
	print STDERR "cpan_guess:3: sep=$separator mod=$module ver=$ver\n" if $verbose;
    } elsif ($module=~s/\.(\d[\d\.]+)$//) {
	$ver=$1;
	$separator='.';
	print STDERR "cpan_guess:4: sep=$separator mod=$module ver=$ver\n" if $verbose;
    } else {
	return;
    }

    if ($KNOWN_EXCEPTIONS{$pkg}) {
	($module, $ver)=@{$KNOWN_EXCEPTIONS{$pkg}};
    } elsif ($ver=~/^(\D\D+)\.([\d\.]+)$/ and $KNOWN_DOT_SEPARATOR{$module.'-'.$1}) {
	$module.=$separator.$1;
	$ver=$2;
    } elsif ($ver=~/^(\D\D+)_V([\d\.]+)$/) {
	$module.=$separator.$1;
	$ver=$2;
    }

    # MARC-Lint_1.47 - todo: merge with above code
    if ($ver=~s/([A-Za-z]+)_(?=\d)//) {
	$module.='-'.$1;
    }
    $ver=&_cpan_trim_version($ver,$module);
    return ($module,$ver);
}

my %DASH_TO_SEP =qw/
SOAP-MIME .
/;

sub _cpan_trim_version {
    my ($ver,$module)=@_;
    # (PERL_SUFFIX_TRIM) code
    #$ver=~s/^[vV]//;
    $ver=~s/^V//;
    $ver=~s/[_-](?:stable|alpha|bis|withoutworldwriteables|source|OpenSource)$//i;
    $ver=~s/-(A|B|C|fix|FIXED|beta6_02|beta13|win32)$/_$1/;
    $ver=~s/(?<=\d)beta[123]$//;
    # 0.02-reupload1
    if ($ver=~/^(.*)-(reupload\d+|[a-z]|\d+.*)$/) {
	my $pref=$1;
	my $suf=$2;
	my $dash_to_sep=$DASH_TO_SEP{$module};
	# TODO: list explicitly all cpanbuilder rpms that use _ and change defaults to .
	$dash_to_sep ||= '_';
	$ver=$pref.$dash_to_sep.$suf;
    }
    $ver=~s/^[vV]//;
    return $ver;
}

1;
